--
--@author Ragman2414
--

SowFertilizer = {}
SowFertilizer.currentModName = g_currentModName;

function SowFertilizer.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(SowingMachine, specializations) and SpecializationUtil.hasSpecialization(Sprayer, specializations)
end;

function SowFertilizer.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", SowFertilizer);
    SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", SowFertilizer);
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", SowFertilizer)
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", SowFertilizer)

end;

function SowFertilizer:onLoad()
    self.sowFertilizerEvents = {};
    self.sowFertilizerEventsIds = {};
    self.isFertilizing = true;
end;

function SowFertilizer:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
    if self.isClient then
        self:clearActionEventsTable(self.sowFertilizerEvents);
        self.sowFertilizerEventsIds = {};
        if isActiveForInputIgnoreSelection then
            local actionEvent, actionEventIdON = self:addActionEvent(self.sowFertilizerEvents, 'ragman_sow_fertilizer', self, SowFertilizer.setFertilizing, false, true, false, true, nil);
            g_inputBinding:setActionEventTextVisibility(actionEventIdON, true);
            g_inputBinding:setActionEventTextPriority(actionEventIdON, GS_PRIO_HIGH);
            table.insert(self.sowFertilizerEventsIds, actionEventIdON);
            self.actionEventIdON = actionEventIdON;
            self.textOff = g_i18n.modEnvironments[SowFertilizer.currentModName].texts["ragman_sow_fertilizer_off"]
            self.textOn = g_i18n.modEnvironments[SowFertilizer.currentModName].texts["ragman_sow_fertilizer_on"]
            g_inputBinding:setActionEventText(self.actionEventIdON, self.textOff)

        end;
    end;
end;

function SowFertilizer:setFertilizing(eventName, actionEvent, fertilizing)

    if(fertilizing == nil) then
        self.isFertilizing = not self.isFertilizing;
    else
        self.isFertilizing = fertilizing
    end

    if( self.isFertilizing )then
        g_inputBinding:setActionEventText(self.actionEventIdON, self.textOff)
    else
        g_inputBinding:setActionEventText(self.actionEventIdON, self.textOn)
    end
end

function SowFertilizer:isFertilizeOn()
    return self.isFertilizing
end

function SowFertilizer.processSowingMachineArea(self,superFunc,superFunc2, workArea, dt)
    local spec = self.spec_fertilizingSowingMachine
    local specSowingMachine = self.spec_sowingMachine
    local specSpray = self.spec_sprayer

    local sprayerParams = specSpray.workAreaParameters
    local sowingParams = specSowingMachine.workAreaParameters
    local changedArea, totalArea

    self.spec_sowingMachine.isWorking = self:getLastSpeed() > 0.5

    if not sowingParams.isActive then
        return 0, 0
    end

    if not self:getIsAIActive() or not g_currentMission.missionInfo.helperBuySeeds then
        if sowingParams.seedsVehicle == nil then
            if self:getIsAIActive() then
                local rootVehicle = self.rootVehicle
                rootVehicle:stopCurrentAIJob(AIMessageErrorOutOfFill.new())
            end

            return 0, 0
        end
    end

    -- stop the sowing machine if the fertilizer tank was filled and got empty
    -- do not stop if the tank was all the time empty
    if not g_currentMission.missionInfo.helperBuyFertilizer then
        if self:getIsAIActive() then
            if sprayerParams.sprayFillType == nil or sprayerParams.sprayFillType == FillType.UNKNOWN then
                if sprayerParams.lastAIHasSprayed ~= nil then
                    local rootVehicle = self.rootVehicle
                    rootVehicle:stopCurrentAIJob(AIMessageErrorOutOfFill.new())
                    sprayerParams.lastAIHasSprayed = nil
                end
            else
                sprayerParams.lastAIHasSprayed = true
            end
        end
    end

    if not sowingParams.canFruitBePlanted then
        return 0, 0
    end

    -- we need to use fertilizer as spraying type because fertilizer is the final blocking value
    local sprayTypeIndex = SprayType.FERTILIZER
    if sprayerParams.sprayFillLevel <= 0 or (spec.needsSetIsTurnedOn and not self:getIsTurnedOn()) or not self.isFertilizing then
        sprayTypeIndex = nil
    end

    local startX,_,startZ = getWorldTranslation(workArea.start)
    local widthX,_,widthZ = getWorldTranslation(workArea.width)
    local heightX,_,heightZ = getWorldTranslation(workArea.height)

    if not specSowingMachine.useDirectPlanting then
        changedArea, totalArea = FSDensityMapUtil.updateSowingArea(sowingParams.seedsFruitType, startX, startZ, widthX, widthZ, heightX, heightZ, sowingParams.fieldGroundType, sowingParams.angle, nil, sprayTypeIndex)
    else
        changedArea, totalArea = FSDensityMapUtil.updateDirectSowingArea(sowingParams.seedsFruitType, startX, startZ, widthX, widthZ, heightX, heightZ, sowingParams.fieldGroundType, sowingParams.angle, nil, sprayTypeIndex)
    end

    self.spec_sowingMachine.isProcessing = self.spec_sowingMachine.isWorking

    if sprayTypeIndex ~= nil then
        local sprayAmount = specSpray.doubledAmountIsActive and 2 or 1
        local sprayChangedArea, sprayTotalArea = FSDensityMapUtil.updateSprayArea(startX, startZ, widthX, widthZ, heightX, heightZ, sprayTypeIndex, sprayAmount)

        sprayerParams.lastChangedArea = sprayerParams.lastChangedArea + sprayChangedArea
        sprayerParams.lastTotalArea = sprayerParams.lastTotalArea + sprayTotalArea
        sprayerParams.lastStatsArea = 0
        sprayerParams.isActive = true
        sprayerParams.lastSprayTime = g_time

        local stats = g_currentMission:farmStats(self:getLastTouchedFarmlandFarmId())
        local ha = MathUtil.areaToHa(sprayerParams.lastChangedArea, g_currentMission:getFruitPixelsToSqm())
        stats:updateStats("sprayedHectares", ha)
        stats:updateStats("sprayedTime", dt/(1000*60))
        stats:updateStats("sprayUsage", sprayerParams.usage)
    end

    sowingParams.lastChangedArea = sowingParams.lastChangedArea + changedArea
    sowingParams.lastStatsArea = sowingParams.lastStatsArea + changedArea
    sowingParams.lastTotalArea = sowingParams.lastTotalArea + totalArea

    -- remove tireTracks
    FSDensityMapUtil.eraseTireTrack(startX, startZ, widthX, widthZ, heightX, heightZ)

    self:updateMissionSowingWarning(startX, startZ)

    return changedArea, totalArea
end
FertilizingSowingMachine.processSowingMachineArea = Utils.overwrittenFunction(FertilizingSowingMachine.processSowingMachineArea, SowFertilizer.processSowingMachineArea )

function SowFertilizer:onReadStream(streamId, connection)
    local fertilizing = streamReadBool(streamId)
    SowFertilizer.setFertilizing(self,nil,fertilizing)
end


function SowFertilizer:onWriteStream(streamId, connection)
    streamWriteBool(streamId, self.isFertilizing)
end

